import { useCallback, useEffect, useState } from "react";
import Select from "../../components/Select";
import Slider from "../../components/Slider";
import { Tool, toolsDatabase } from "../../toolsDatabase";

const dropdowns = [
	{
		name: "rodzajFrezowania",
		label: "Rodzaj Frezowania",
		options: ["Czolowe", "Walcowo Czołowe", "Rowkowe"],
	},
	{
		name: "rodzajObrobki",
		label: "Rodzaj Obróbki",
		options: ["Zgrubna", "Wykończeniowa"],
	},
	{
		name: "material",
		label: "Materiał",
		options: ["Stal węglowa", "Stal nierdzewna", "Żeliwo"],
	},
];
interface IInput {
	rodzajFrezowania: number;
	rodzajObrobki: number;
	material: number;
	glebokoscSkrawania: number;
	szerokoscSkrawania: number;
	tolerancja: number;
	chropowatosc: number;
}
// interface IResults {
// 	predkoscObrotowaWrzeciona: number;
// 	predkoscSkrawania: number;
// 	predkoscPosuwu: number;
// 	posowNaOstrze: number;
// 	posowNaObrot: number;
// 	wydajnoscObjetosciowa: number;
// }

const HomePage = () => {
	// ----------------------------- STATE -----------------------------
	const [input, setInput] = useState<IInput>({
		// lists
		rodzajFrezowania: 0,
		rodzajObrobki: 0,
		material: 0,
		// variables
		glebokoscSkrawania: Math.floor(Math.random() * 50 + 1),
		szerokoscSkrawania: Math.floor(Math.random() * 50 + 1),
		tolerancja: parseFloat((0.1 + Math.random() * (5 - 0.1)).toFixed(2)),
		chropowatosc: parseFloat((0.1 + Math.random() * (5 - 0.1)).toFixed(2)),
	});
	// const [calcResults, setCalcResults] = useState<IResults>({
	// 	// constans
	// 	predkoscObrotowaWrzeciona: 2700,
	// 	predkoscSkrawania: 170,
	// 	predkoscPosuwu: 1140,
	// 	posowNaOstrze: 0.14,
	// 	posowNaObrot: 0.42,
	// 	wydajnoscObjetosciowa: 114,
	// });
	const [result, setResult] = useState<Tool | null>(null);
	const [calculating, setCalculating] = useState(false);
	const [notFound, setNotFound] = useState(false);
	// ----------------------------- FUNCTION --------------------------
	const handleChangeOfField = (key: keyof typeof input) => (e: React.ChangeEvent<HTMLSelectElement>) => {
		const value = Number(e.target.value);
		setInput((prev) => ({ ...prev, [key]: value }));
	};
	const matchesInput = (tool: Tool, input: IInput): boolean => {
		return (
			tool.rodzajFrezowania === input.rodzajFrezowania &&
			tool.rodzajObrobki === input.rodzajObrobki &&
			tool.material === input.material &&
			input.chropowatosc >= tool.chropowatosc[0] &&
			input.chropowatosc <= tool.chropowatosc[1] &&
			input.szerokoscSkrawania >= tool.szerokoscSkrawania[0] &&
			input.szerokoscSkrawania <= tool.szerokoscSkrawania[1] &&
			input.glebokoscSkrawania >= tool.glebokoscSkrawania[0] &&
			input.glebokoscSkrawania <= tool.glebokoscSkrawania[1] &&
			input.tolerancja >= tool.tolerancja[0] &&
			input.tolerancja <= tool.tolerancja[1]
		);
	};
	const checkInputInDatabse = useCallback((input: IInput): void => {
		console.debug("checking...", input);

		const foundTool = toolsDatabase.tools.find((tool) => matchesInput(tool, input));

		if (foundTool) {
			console.debug("found", foundTool);
			setResult(foundTool);
		} else {
			console.debug("not found");
			setResult(null);
			setNotFound(true);
		}
	}, []);

	// ----------------------------- EFFECT ----------------------------
	useEffect(() => {
		if (calculating) {
			checkInputInDatabse(input);
		}
		setCalculating(false);
	}, [calculating, checkInputInDatabse, input]);
	// ----------------------------- RENDER ----------------------------
	return (
		<>
			<div className="p-6 bg-white rounded shadow-lg lg:w-[500px] space-y-4 mb-4 lg:ml-[50%] lg:translate-x-[-50%]">
				<h1 className="text-3xl mb-6 text-center text-gray-700">Wybór parametrów</h1>
				<div className="space-y-4">
					{dropdowns.map((dropdown) => (
						<div className="mb-4" key={dropdown.label}>
							<label className="block text-gray-700 text-sm font-bold mb-2">{dropdown.label}</label>
							<Select
								value={input[dropdown.label as keyof typeof input]}
								onChange={handleChangeOfField(dropdown.name as keyof typeof input)}
								options={dropdown.options}
								label={dropdown.label}
								disabledLabel={`Wybierz ${dropdown.label}`}
							/>
						</div>
					))}
				</div>

				<label className="flex text-gray-700 text-sm font-bold mb-0">Głębokość Skrawania</label>
				<Slider
					range={{ min: 1, max: 50 }}
					value={input.glebokoscSkrawania}
					onChange={(e) => {
						setInput((prev) => ({ ...prev, glebokoscSkrawania: Number(e!.target.value) }));
					}}
					suffix="mm"
				/>

				<label className="flex text-gray-700 text-sm font-bold mb-0">Szerokość Skrawania</label>
				<Slider
					range={{ min: 1, max: 50 }}
					value={input.szerokoscSkrawania}
					onChange={(e) => {
						setInput((prev) => ({ ...prev, szerokoscSkrawania: Number(e!.target.value) }));
					}}
					suffix="mm"
				/>

				<label className="flex text-gray-700 text-sm font-bold mb-0">Tolerancja</label>
				<Slider
					range={{ min: 0.1, max: 5 }}
					value={input.tolerancja}
					onChange={(e) => {
						setInput((prev) => ({ ...prev, tolerancja: Number(e!.target.value) }));
					}}
					suffix="mm"
					step={0.01}
				/>

				<label className="flex text-gray-700 text-sm font-bold mb-0">Chropowatość</label>
				<Slider
					range={{ min: 0.1, max: 5 }}
					value={input.chropowatosc}
					onChange={(e) => {
						setInput((prev) => ({ ...prev, chropowatosc: Number(e!.target.value) }));
					}}
					suffix="mm"
					step={0.01}
				/>

				<button
					className="btn btn-secondary w-full mt-4"
					type="submit"
					onClick={() => {
						setCalculating(true);
					}}
				>
					Policz
				</button>
			</div>

			<div className="p-6 bg-white rounded shadow-lg lg:w-[500px] mb-4 lg:ml-[50%] lg:translate-x-[-50%]">
				{result ? (
					<>
						<h2 className="text-2xl mb-4">{result.name}</h2>
						<p className="text-lg">
							Płyta: <span className="text font-mono font-semibold">{result.plate}</span>
						</p>
						<p className="text-lg">
							Ilość zębów: <span className="text font-mono font-semibold">{result.ilosc_zebow}</span>
						</p>
					</>
				) : notFound ? (
					<h2 className="text-xl">❗Nie znaleziono narzędzia dla podanych parametrów</h2>
				) : (
					<p className="text-neutral-content">Tutaj pojawią się wyniki</p>
				)}
			</div>
		</>
	);
};

export default HomePage;
